require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  MUTATE_PARAMS = {
      title: 'String',
      siret: 'String',
      siren: 'String',
      is_root: 'Boolean',
      root_id: 'String',
      api_id: 'String',
      rpls_id: 'String',
      affiliate: 'String',
  }

  ACCEPTABLE_PARAMS = %w[
    created_at
    updated_at
    title
    siret
    siren
    is_root
    root_id
    api_id
    rpls_id
    affiliate
    housing_offer_ids
    lessor_objective_ids
    group_id
    supervisor_group_id
    manager_group_id
  ]
  OBJECTIVES_ACCEPTABLE_PARAMS = %w[
    created_at
    updated_at
    activation_date
    department
    district
    threshold_type
    priority_public
    new_offer
    target_by_typologies
    realized_by_typologies
    present_typologies
    agency_id
  ]

  VALID_OPERATORS = %w[
    $in
    $nin
    $ne
    $gt
    $gte
    $lt
    $lte
    $eq
    $regex
    $all
  ]

  OBJECTIVES_REGEX_PARAMS = %w[
    district
  ]

  describe Api::Dmo::LessorsController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper
    let!(:responsible) {create(:user, roles: ['Administrator'])}
    let!(:user) {create(:user)}

    #TESTS BASIQUES D'API

    describe 'POST /lessors' do
      describe 'Create lessor with minimal valid datas' do
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
              data: {
                  attributes: {
                      user_id: responsible.id.to_s
                  }
              }
          }
        end
        it 'returns status code 201' do
          expect(response).to have_http_status(201)
        end
      end
      MUTATE_PARAMS.each do |param, type|
        describe 'post request with ' + param.to_s + ' (' + type.to_s + ') is valid' do
          let(:params) do
            attr = {}
            attr[param] = get_typed_value(type)
            attr
          end
          before :each do
            params['user_id'] = user.id.to_s
            authenticated_header(request, responsible)
            post 'create', {
                data: {
                    attributes: params
                }
            }
          end
          it 'returns status code 201 and the value appears in json' do
            expect(response).to have_http_status(201)
            if (type == 'Date' || type == 'DateTime')
              expect(json['data']['attributes'][param.to_s]).not_to be_empty
              expect(json['data']['attributes'][param.to_s]).not_to eq('')
            else
              expect(json['data']['attributes'][param.to_s]).to eq(get_typed_value(type))
            end
          end
        end
      end
      #housing_offer_ids:'Array',
      describe 'post request with housing_offer_ids' do
        let!(:housing_offer1) {create(:housing_offer, description: 'acceuillant', terraces_or_balconies: 50)}
        let!(:housing_offer2) {create(:housing_offer, description: 'Super', terraces_or_balconies: 150)}
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
              data: {
                  attributes: {
                      user_id: responsible.id.to_s,
                      housing_offer_ids: [housing_offer1.id.to_s, housing_offer2.id.to_s]
                  }
              }
          }
        end
        it 'returns status code 201 and the value appears in json' do
          expect(response).to have_http_status(201)
          expect(json['data']['attributes']['housing_offer_ids']).to eq([housing_offer1.id.to_s, housing_offer2.id.to_s])
        end
      end
      #lessor_objective_ids: 'Array'
      describe 'post request with lessor_objective_ids' do
        let!(:lessor_objective1) {create(:lessor_objective, department: 'acceuillant', user_id: responsible.id.to_s)}
        let!(:lessor_objective3) {create(:lessor_objective, department: 'Super', user_id: responsible.id.to_s)}
        let!(:lessor_objective2) {create(:lessor_objective, department: 'bel appart', user_id: responsible.id.to_s)}


        before :each do
          authenticated_header(request, responsible)
          post 'create', {
              data: {
                  attributes: {
                      user_id: responsible.id.to_s,
                      lessor_objective_ids: [lessor_objective1.id.to_s, lessor_objective2.id.to_s]
                  }
              }
          }
        end
        it 'returns status code 201 and the value appears in json' do
          expect(response).to have_http_status(201)
          expect(json['data']['attributes']['lessor_objective_ids']).to eq([lessor_objective1.id.to_s, lessor_objective2.id.to_s])
        end
      end
    end

    describe 'GET /lessors' do
      describe 'GET /lessors' do
        let!(:lessor1) {create(:lessor, user_id: responsible.id.to_s)}
        let!(:lessor3) {create(:lessor, user_id: responsible.id.to_s)}
        let!(:lessor2) {create(:lessor, user_id: responsible.id.to_s)}
        before :each do
          authenticated_header(request, responsible)
          get 'index'
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(3)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(3)
        end
      end

      describe 'GET /lessors' do
        let!(:lessor1) {create(:lessor, user_id: responsible.id.to_s)}
        let!(:lessor3) {create(:lessor, user_id: responsible.id.to_s)}
        let!(:lessor2) {create(:lessor, user_id: responsible.id.to_s)}
        before :each do
          authenticated_header(request, responsible)
          get 'index', {ids: [lessor1.id.to_s, lessor2.id.to_s]}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(2)
        end
      end

      ACCEPTABLE_PARAMS.each do |acceptable_param|
        type = MUTATE_PARAMS[acceptable_param.to_sym] || MUTATE_PARAMS[acceptable_param.to_s]
        if type
          describe 'GET /lessors with sort on field ' + acceptable_param + ' works ' do
            let!(:lessor3) {create(:lessor, acceptable_param => get_typed_value(type, 2), user_id: responsible.id.to_s)}
            let!(:lessor1) {create(:lessor, acceptable_param => get_typed_value(type, 0), user_id: responsible.id.to_s)}
            let!(:lessor2) {create(:lessor, acceptable_param => get_typed_value(type, 1), user_id: responsible.id.to_s)}
            before :each do
              authenticated_header(request, responsible)
              get 'index', {sort: {acceptable_param => 1}}
            end
            it 'returns sorted results' do
              expect(response.status).to eq(200)
              if type == 'DateTime'
                lessor_attribute = json['data'].map {|unit| DateTime.parse(unit['attributes'][acceptable_param])}
                expect(lessor_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
              elsif type == 'Date'
                lessor_attribute = json['data'].map {|unit| Date.parse(unit['attributes'][acceptable_param])}
                expect(lessor_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
              else
                lessor_attribute = json['data'].map {|unit| unit['attributes'][acceptable_param]}
                expect(lessor_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
              end
            end
          end
        end
      end
      describe 'GET /lessors with operators' do
        let!(:lessor1) {create(:lessor, title: 'accueillant', rpls_id: '123', user_id: responsible.id.to_s)}
        let!(:lessor3) {create(:lessor, title: 'Super', rpls_id: '123', user_id: responsible.id.to_s)}
        let!(:lessor2) {create(:lessor, title: 'bel appart', rpls_id: '123', user_id: responsible.id.to_s)}
        let!(:lessor4) {create(:lessor, title: 'nothing', rpls_id: '456', user_id: responsible.id.to_s)}
=begin
      describe 'with $lt' do
        before :each do
          authenticated_header(request, responsible)
          get 'index', {documents_required: {'$lt': 100}}
        end
        it 'returns results' do
          expect(response.status).to eq(200)
          expect(json['meta']['pagination']['total_objects']).to eq(1)
        end
      end
=end

        before :each do
          authenticated_header(request, responsible)
        end

        describe 'with $in' do
          before :each do
            get 'index', {title: {'$in': ['accueillant', 'flamand']}}
          end
          it 'returns results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
        describe 'with $ne' do
          before :each do
            get 'index', {title: {'$ne': 'accueillant'}}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(3)
          end
        end
        describe 'with $regex' do
          before :each do
            get 'index', {title: {'$regex': /^S/}}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
        describe 'with $or' do
          before :each do
            get 'index', {'$or' => [{'title' => 'accueillant'}, {'title' => 'Super'}]}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(2)
          end
        end
        describe 'with $or and one param is invalid but within single hash' do
          before :each do
            get 'index', {'$or' => [{'title' => 'chétif', 'rpls_id' => '456'}]}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(0)
          end
        end
        describe 'with $or and one param is invalid within separated hash' do
          before :each do
            get 'index', {'$or' => {'0' => {'title' => 'chétif'}, '1' => {'rpls_id' => '456'}}}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
        describe 'with matching $and' do
          before :each do
            get 'index', {'$and' => [{'title' => {'$ne' => 'Super'}}, {'rpls_id' => '123'}]}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(2)
          end
        end
        describe 'with not matching $and' do
          before :each do
            get 'index', {'$and' => [{'title' => 'accueillant'}, {'rpls_id' => {'$ne' => '123'}}]}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(0)
          end
        end
      end
    end

    describe 'GET /lessors/:id' do
      describe 'verif it work' do
        let!(:lessor1) {create(:lessor, title: 'acceuillant', user_id: responsible.id.to_s)}
        before :each do
          authenticated_header(request, responsible)
          get 'show', {id: lessor1.id}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
        end
      end
      describe 'verif it work not work with a non authorized user' do
        let!(:lessor1) {create(:lessor, user_id: responsible.id.to_s)}

        before :each do
          authenticated_header(request, user)
          get 'show', {id: lessor1.id}
        end
        it 'returns status code 404' do
          expect(response).to have_http_status(404)
        end
      end
    end

    describe 'PATCH /lessors/:id' do
      lessor_params = {}
      MUTATE_PARAMS.each do |param, type|
        lessor_params[param] = get_typed_value(type)
      end

      let!(:lessor) {create(:lessor, **lessor_params, user_id: responsible.id.to_s)}
      MUTATE_PARAMS.each do |param, type|
        describe 'patch request with ' + param.to_s + ' ' + type.to_s + 'is valid' do
          let(:params) do
            attr = {}
            attr[param] = get_typed_value(type, 2)
            attr
          end
          before :each do

            authenticated_header(request, responsible)
            patch 'update', {
                id: lessor.id.to_s,
                data: {
                    attributes: params
                }
            }
          end
          it 'returns status code 200 and the value appears in json' do
            expect(response).to have_http_status(200)
            if (type == 'Date' || type == 'DateTime')

              expect(json['data']['attributes'][param.to_s]).not_to be_empty
              expect(json['data']['attributes'][param.to_s]).not_to eq('')
            else
              expect(json['data']['attributes'][param.to_s]).to eq(get_typed_value(type, 2))
            end
          end
        end
      end

      describe 'DELETE /lessors/:id' do
        let!(:lessor) {create(:lessor, user_id: responsible.id.to_s)}
        before :each do
          authenticated_header(request, responsible)

          delete 'destroy', {
              id: lessor.id.to_s
          }
        end
        it 'returns status code 204' do
          expect(response).to have_http_status(204)
        end
      end

    end

    #TESTS DE CHARLY
    describe 'more complex requests' do

      let!(:lessor) {create(:lessor, user_id: responsible.id, title: 'Lessor')}
      let!(:lessor2) {create(:lessor, user_id: responsible.id, title: 'Lessor 2')}
      let!(:lessor3) {create(:lessor, user_id: responsible.id, title: 'Lessor 3')}
      let!(:lessor4) {create(:lessor, user_id: user.id, title: 'Lessor 4')}

      describe 'GET /lessors' do

        before :each do
          authenticated_header(request, responsible)
          get 'index'
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty

          expect(json['data'].length).to eq(4)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(4)
        end
      end

      describe 'GET /lessors with responsible' do
        before :each do
          authenticated_header(request, responsible)
          get 'index', responsible: responsible.id
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty

          expect(json['data'].length).to eq(3)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(3)
        end
      end

      # describe 'GET /lessors with sort' do
      #   before :each do
      #     authenticated_header(request, responsible)
      #     get 'index', {sort: 'title:-1'}
      #   end
      #
      #   it 'returns sorted results' do
      #
      #     expect(response.status).to eq(200)
      #     lessor_titles = json['data'].map{ |unit| unit['attributes']['title'] }
      #     expect(lessor_titles).to eq([lessor3.title, lessor2.title, lessor.title])
      #   end
      # end
      describe 'GET /lessors/:id/current_operation_names' do
        let!(:housing_offer1) {create(:housing_offer, description: 'acceuillant', terraces_or_balconies: 50, operation_name: "AAAA", lessor_id:lessor.id)}
        let!(:housing_offer2) {create(:housing_offer, description: 'Super', terraces_or_balconies: 150, operation_name: "BBBB", lessor_id:lessor.id)}

        context 'as responsible' do
          before :each do

            authenticated_header(request, responsible)
            get 'current_operation_names', {id: lessor.id}
          end

          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end

          it 'returns valid JSON' do
           ap json
            expect(json).to eq(["AAAA", "BBBB"])

          end
        end
      end
      describe 'GET /lessors/:id' do

        context 'as responsible' do
          before :each do
            authenticated_header(request, responsible)
            get 'show', {id: lessor.id}
          end

          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end

          it 'returns valid JSON' do
            subject
            expect(json['data']['id']).to eq(lessor.id.to_s)
            #expect(json['data']['attributes']['read_level']).to eq('write')
            expect(json['data']['attributes']['title']).to eq(lessor.title)
          end
        end

        context 'with inaccessible lessor' do
          before :each do
            authenticated_header(request, responsible)
            get 'show', {id: 'test_unavailable_lessor'}
          end

          it 'returns status code 404' do
            expect(response).to have_http_status(404)
          end
        end
      end

      describe 'PUT /lessors/:id' do
        describe 'update my lessor' do
          #let(:valid_params) { ams_json(Lessor, responsible, {id: lessor.id, superfields: {'key': 'value'}}) }
          let(:valid_params) {{id: lessor.id.to_s, data: {attributes: {title: 'lessor u'}}}}

          before :each do
            authenticated_header(request, responsible)
            put 'update', valid_params
          end

          it 'returns status code 200' do
            expect(response.status).to eq(200)
          end

          it 'updated form' do
            expect(json['data']['id']).to eq(lessor.id.to_s)
            #expect(json['data']['attributes']['read_level']).to eq('write')
            expect(json['data']['attributes']['title']).to eq('lessor u')
          end
        end
      end

      describe 'Users associations' do
        let!(:lessor_user) { create(:user, roles: ['Bailleur']) }
        let!(:lessor_user_for_associate) { create(:user) }
        let!(:lessor) { create(:lessor, user_id: lessor_user.id) }
        let(:valid_params) {{id: lessor.id.to_s, data: {attributes: {user_id: lessor_user_for_associate.id.to_s}}}}
  
        before :each do
          authenticated_header(request, lessor_user)
        end
        
        it 'can add user to lessor' do
          put :add_user, valid_params
          expect(response.status).to eq(200)
          expect(json['data']).to eq('user updated')
          lessor_user_for_associate.reload
          expect(lessor_user_for_associate.lessor).to eq(lessor)
        end
        
        it 'can remove user to lessor' do
          lessor_user_for_associate.update(lessor_id: lessor.id)
          lessor_user_for_associate.reload
          expect(lessor_user_for_associate.lessor).to eq(lessor)
          
          put :remove_user, valid_params
          expect(response.status).to eq(200)
          expect(json['data']).to eq('user updated')
          lessor_user_for_associate.reload
          expect(lessor_user_for_associate.lessor).to be_nil
        end
      end
    end
  end

  require_relative '../../swagger_helper'

  describe Hermes::Api::Dmo::LessorsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_LESSORS = 'API_DMO_LESSORS'
    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
    let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}

    context 'as admin' do
      let!(:admin) {create(:administrator)}

      path '/api/dmo/lessors' do
        post 'Create lessor' do
          tags TAGS_API_DMO_LESSORS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          json_params[:user_id] = {type: :string}
          json_params[:housing_offer_ids] = {type: :array}
          json_params[:lessor_objective_ids] = {type: :array}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '201', 'lessor created' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:params) do
              jsonapi_attributes({}
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'lessors'
            end
          end
        end
      end

      path '/api/dmo/lessors' do
        get 'Retrieves all lessors' do
          tags TAGS_API_DMO_LESSORS
          produces 'application/json'

          parameter name: 'ids[]', in: :query, type: :array, items: {type: :string}, collectionFormat: :multi, required: false
          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = ''
            if MUTATE_PARAMS.include?(acceptable_param)
              type = nice_type(MUTATE_PARAMS[acceptable_param])
            end
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end

          # VALID_OPERATORS.each do |valid_operator|
          #   parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
          #end

          security [Bearer: {}]
          paginator

          response '200', 'lessors found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let!(:lessors) {create_list(:lessor, 5)}

            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) {lessors}
            end
          end
        end
      end

      path '/api/dmo/lessors/{id}' do
        get 'Retrieves a single lessor by id' do
          tags TAGS_API_DMO_LESSORS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '200', 'lessor found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:lessor) {create(:lessor)}
            let(:id) {lessor.id}

            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) {lessor}
            end
          end
        end
      end

      path '/api/dmo/lessors/{id}/all_objectives' do
        get 'Retrieves all housing objectives relatives to a lessor' do
          tags TAGS_API_DMO_LESSORS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          parameter name: 'ids[]', in: :query, type: :array, items: {type: :string}, collectionFormat: :multi, required: false
          security [Bearer: {}]
          OBJECTIVES_ACCEPTABLE_PARAMS.each do |acceptable_param|
            parameter name: acceptable_param.to_s, in: :query, required: false
          end
          paginator

          response '200', 'housing_objectives found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:lessor) {create(:lessor)}
            let(:id) {lessor.id}

            include_context 'not containings errors'
            run_test!
          end
        end
      end

      path '/api/dmo/lessors/{id}/current_operation_names' do
        get 'Retrieves all operation_name of housing_offers relatives to a lessor' do
          tags TAGS_API_DMO_LESSORS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]


          response '200', 'operation name found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:lessor) {create(:lessor)}
            let(:id) {lessor.id}
            let!(:housing_offer1) {create(:housing_offer, description: 'acceuillant', terraces_or_balconies: 50, operation_name: "AAAA")}
            let!(:housing_offer2) {create(:housing_offer, description: 'Super', terraces_or_balconies: 150, operation_name: "AAAA")}

            run_test!

          end
        end
      end

      path '/api/dmo/lessors/{id}/active_objectives' do
        get 'Retrieves all active housing objectives relatives to a lessor' do
          tags TAGS_API_DMO_LESSORS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          parameter name: 'ids[]', in: :query, type: :array, items: {type: :string}, collectionFormat: :multi, required: false
          security [Bearer: {}]
          OBJECTIVES_ACCEPTABLE_PARAMS.each do |acceptable_param|
            parameter name: acceptable_param.to_s, in: :query, required: false
          end
          paginator

          response '200', 'housing_objectives found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:lessor) {create(:lessor)}
            let(:id) {lessor.id}

            include_context 'not containings errors'
            run_test!
          end
        end
      end

      path '/api/dmo/lessors/{id}' do
        put 'edit lessor/{id}' do
          tags TAGS_API_DMO_LESSORS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          json_params[:housing_offer_ids] = {type: :array}
          json_params[:lessor_objective_ids] = {type: :array}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end

          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '200', 'lessor created' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:lessor) {create(:lessor)}
            let(:id) {lessor.id}
            let(:params) do
              jsonapi_attributes(
               {title: 'String'} 

              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'
            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'lessors'
            end
          end
        end
      end

      path '/api/dmo/lessors/{id}' do
        delete 'Delete a single lessor by id' do
          tags TAGS_API_DMO_LESSORS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '204', 'lessor deleted' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:id) {create(:lessor, responsible: admin.id).id}
            run_test!
          end
        end
      end
    end
  end
end